/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: midiChopimg.c,v 1.3 2012/08/08 20:43:22 tristram Exp $
 * $Log: midiChopimg.c,v $
 * Revision 1.3  2012/08/08 20:43:22  tristram
 * Corrected warning messages when compiling the programmes.
 * Use entire spectral range for average subtraction in oirFormFringes.
 *
 * Revision 1.2  2005/08/08 09:56:26  koehler
 * Removed GPL-blurb (will be added by Makefile.export)
 *
 * Revision 1.1  2005/01/19 14:17:42  jaffe
 * new heidelberg additions
 *
 *
 * midiChopimg.c
 *
 * Created:     Sun Aug  3 13:18:57 2003 by Koehler@rommie
 * Last change: Mon Aug  8 11:55:13 2005
 *
 * Based on oirCompressData.c by Walter Jaffe
 *
 *
 * USAGE: midiChopimg [-b before] [-a after] "midi-files" outfile
 */

#undef DEBUG
#define DEBUG 1

#define MAX_REGIONS 4	/** max. number of data-regions we can handle **/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include "midiMultifile.h"


/********************************* MAIN *********************************/

int main( int argc, char **argv)
{
   struct multifile *MFile;

   int nRegion, iRegion;
   int maxstep;
   int i,ierr;
   long iRow, oRow;
   int nx, ny;
   long npix, ipix;
   int iKey;

   fitsfile *inFile;
   fitsfile *outFile = NULL;
   imaging_data_table_def *idata;
   imaging_fdata_table_def *odata;
   imaging_detector_table_def *outDetectorDef;
   imaging_data_row **inData;
   imaging_fdata_row *outDataT, *outDataS;

  int arg;
  int before=1, after=1, tatyidx=-1;
  short verbose = 0;
  long nBuf, iBuf, nTarg=0, nSky=0;
  int past, present, future;

  for (arg=1;  arg<argc;  ++arg)
  {
    if (argv[arg][0]=='-' && arg+1 < argc)
    {
	if      (argv[arg][1]=='b') { before= atoi(argv[++arg]); }
	else if (argv[arg][1]=='a') { after = atoi(argv[++arg]); }
	else if (argv[arg][1]=='t') { tatyidx= atoi(argv[++arg]); }
	else if (argv[arg][1]=='v') { verbose= 1; }
	else if (argv[arg][1]=='-') { ++arg;  break; }
    }
    else break;
  }
  if( arg+1 >= argc )
  {  printf("USAGE: %s [-before #] [-after #] inputfile outputfile\n",argv[0]);
	return 10;
  }
  printf("before: %d, after: %d frames\n",before,after);

  ierr = 0;
  if ((MFile = multifile_open(argv[arg], &ierr)) == NULL)
  {	fits_report_error(stdout,ierr);  return 11;  }

  /*
   *    open first file and get detector info; then close file
   */
  fits_open_file (&inFile, MFile->filelist[0], READONLY, &ierr);
  if (ierr != 0)
  {  fits_report_error(stdout,ierr);  return 12;  }

  outDetectorDef = read_imaging_detector_table(inFile, &ierr);
  idata          = open_imaging_data_table(inFile, &ierr);

  nBuf = after + 1 + before;	/** look "after" frames in the past,
				    "before" frames into the future,
				    and 1 frame for the present  **/
  inData = malloc(sizeof(imaging_data_row *) * nBuf);
  for (iBuf=0;  iBuf<nBuf;  ++iBuf)
	inData[iBuf] = make_imaging_data_row(idata);
  nRegion        = idata->nregion;
  iKey = table_find_key(idata->def, "MAXSTEP");
  if (iKey>0)
	table_get_keyword(idata->def, iKey, (void*)&maxstep, &ierr);

  fits_close_file(inFile, &ierr);
  kill_imaging_data_table_def(idata,1);

#ifdef DEBUG
   fprintf(stderr,"got detector info from 1.file\n");
#endif
  /*
   *    create output data table
   */
   fits_create_file (&outFile, argv[arg+1], &ierr);
   if (ierr !=0)
   {  fits_report_error(stderr, ierr);  return 13; }

   odata= make_imaging_fdata_table_def(outFile, outDetectorDef, maxstep, &ierr);
   if (ierr !=0)
   {  fits_report_error(stderr, ierr);  return 14; }

   outDataT = make_imaging_fdata_row(odata);
   outDataS = make_imaging_fdata_row(odata);
   oRow = 1;

#ifdef DEBUG
   fprintf(stderr,"output data table created.\n");
#endif
  /*
   *  set data and nonData info to some defined state
   */
   outDataT->time	 = outDataS->time	 = -1.;
   outDataT->opd[0]	 = outDataS->opd[0]	 = -1.;
   outDataT->opd[1]	 = outDataS->opd[1]	 = -1.;
   outDataT->local_opd[1]= outDataS->local_opd[1]= -1.;
   outDataT->local_opd[0]= outDataS->local_opd[0]= -1.;
   outDataT->exp	 = outDataS->exp	 = -1.;
   outDataT->offset[0]	 = outDataS->offset[0]	 = -1.;
   outDataT->offset[1]	 = outDataS->offset[1]	 = -1.;
   outDataT->rotation	 = outDataS->rotation	 = -1.;
   outDataT->frame	 = outDataS->frame	 = 0;
   outDataT->reference	 = outDataS->reference	 = 0;

   for (iRegion=0; iRegion<nRegion; iRegion++)
   {
     npix = (long) outDetectorDef->naxis[iRegion][0]
	  * (long) outDetectorDef->naxis[iRegion][1];
     /* fprintf(stderr,"init region %d - %ld pixels\n",iRegion,npix); */

     outDataT->targtype[iRegion] = 'T';
     outDataS->targtype[iRegion] = 'S';
     for( ipix=0;  ipix<npix;  ++ipix)
     {
	outDataT->data[iRegion][ipix]= 0.0;
	outDataS->data[iRegion][ipix]= 0.0;
     }
   }
#ifdef DEBUG
   fprintf(stderr,"output data table initialized\n");
#endif

  if(tatyidx<0) {
    /** user didn't specify which targtype to use **/
    tatyidx = (nRegion==4);	/** 1 in sci-phot mode, 0 otherwise **/
#ifdef DEBUG
    fprintf(stderr,"using targtype #%d\n",tatyidx);
#endif
  }

 /***********************************************************************
  *     loop over all frames alias rows
  ***********************************************************************/
  /** rows in fbts are 1-based! **/

  /** first fill the buffer **/
  for( iRow=1, iBuf=0;  iBuf < nBuf;  ++iRow, ++iBuf)
  { multifile_rd_img_data_tbl( MFile, inData[iBuf], iRow, &ierr);
    /* printf("%5ld: tartyp %c\n",iRow, inData[iBuf]->targtype[0]); */
  }
  if (ierr != 0)
  {	fits_report_error(stderr,ierr);
	multifile_close(MFile, &ierr);
	return 20;
  }
  past= 0;  present= past+after;  future= present+before;

  for( iBuf=0;  iRow <= MFile->totframes;  iRow++)
  {
    char tartyp;

    if (verbose)
	printf("%5ld: tartyp %c %c %c => ",
		iRow-before-1,	   inData[past]->targtype[tatyidx],
				inData[present]->targtype[tatyidx],
				 inData[future]->targtype[tatyidx] );

    tartyp = inData[present]->targtype[tatyidx];
    if (tartyp != 'U')
    { for (i=past;  i != present;  i= (i+1)%nBuf)
	if (inData[i]->targtype[tatyidx] != tartyp) { tartyp='A'; break; }

      if (tartyp!='A')
	for (i= (present+1)%nBuf;  ;  i= (i+1)%nBuf)
	{ if (inData[i]->targtype[tatyidx] != tartyp)  { tartyp='B'; break; }
          if (i==future) break;
	}
    }
    if (verbose)
      switch (tartyp)
      {	case 'A': printf("after move\n");  break;
	case 'B': printf("before move\n"); break;
	case 'T': printf("on target\n");   break;
	case 'S': printf("on sky\n");	   break;
	case 'U': printf("moving\n");	   break;
	default:  printf("unknown tartyp\n");
      }

    if (tartyp == 'T' || tartyp == 'S')
    { for (iRegion=0; iRegion<nRegion; iRegion++)
      {
	short *inPix;
	float *outPix;

	nx = MFile->inDetectorDef->naxis[iRegion][0];
        ny = MFile->inDetectorDef->naxis[iRegion][1];
        npix = nx * (long)ny;
        inPix = (short *)inData[present]->data[iRegion];
	if (tartyp == 'T')
		outPix= outDataT->data[iRegion];
	else	outPix= outDataS->data[iRegion];

	for (ipix=0; ipix<npix; ipix++)
		*(outPix++) += 32768. + (float) *(inPix++);
      }
      if (tartyp == 'T') { ++nTarg; }  else { ++nSky; }
    }

    if( (multifile_rd_img_data_tbl( MFile, inData[iBuf], iRow, &ierr) != 0))
    {	fits_report_error(stderr,ierr);
	multifile_close(MFile, &ierr);
	return 20;
    }
    iBuf   = (iBuf   +1)%nBuf;
    past   = (past   +1)%nBuf;
    present= (present+1)%nBuf;
    future = (future +1)%nBuf;
  }
  printf("%ld good frames on target, %ld good frames on sky\n",nTarg,nSky);

  for (iRegion=0; iRegion<nRegion; iRegion++)
  {
	float nT= (float)nTarg, nS= (float)nSky;

	nx = MFile->inDetectorDef->naxis[iRegion][0];
	ny = MFile->inDetectorDef->naxis[iRegion][1];
	npix = nx * (long)ny;

	for( ipix=0; ipix<npix;  ++ipix)  /** foreach pix in region **/
	{
	  outDataT->data[iRegion][ipix] /= nT;
	  outDataS->data[iRegion][ipix] /= nS;
	}
  }
  write_imaging_fdata_table(outFile, odata, outDataT, oRow++, &ierr);
  write_imaging_fdata_table(outFile, odata, outDataS, oRow++, &ierr);

  if( close_imaging_fdata_table(outFile, odata, &ierr))
	fits_report_error(stderr,ierr);

  multifile_close(MFile,&ierr);
  kill_imaging_fdata_row(outDataT);
  kill_imaging_fdata_row(outDataS);
  for (iBuf=0;  iBuf<nBuf;  ++iBuf)
	kill_imaging_data_row(inData[iBuf]);
  free(inData);

  return 0;
}
